@@ -38,7 +38,7 @@ nonpar_trawlest <- function(data, Delta, lag=100){
   #Cover the case when lag==n and remove the last NA value
   n <- base::length(data) #it includes time points 0, Delta, ...(n-1) Delta
   if(lag==n){
-    a_hat <- a_hat[1:n-1]
+    a_hat <- a_hat[1:(n-1)]
   }
 
 
@@ -50,6 +50,58 @@ nonpar_trawlest <- function(data, Delta, lag=100){
   return(list("a_hat"=a_hat, "a0_alt"=a0_alt))
 }
 
+
+#######################
+#'This function implements bias correction for the nonparametric trawl
+#'estimation proposed in Sauri and Veraart (2022).
+#'@title Bias-corrected nonparametric estimation of the trawl function
+#'@name trawl_est
+#'@param data Data to be used in the trawl function estimation.
+#'@param Delta Width of the grid on which we observe the data
+#'@details This function performs bias correction on the nonparametric trawl
+#'function estimates obtained using the methodology of Sauri and Veraart (2022).
+#'The bias correction is implemented by estimating the derivative of the trawl
+#'function and applying the correction \eqn{0.5 \times \Delta \times
+#'\hat{a}'(t)},
+#'where \eqn{\hat{a}'(t)} is the estimated derivative at lag \eqn{t}.
+#'
+#'Suppose the data is observed on the grid
+#'0, Delta, 2Delta, ..., (n-1)Delta. Given the path contained in data,
+#'the function returns both the original and bias-corrected estimates of the
+#'trawl function.
+#'
+#'The bias correction is based on the theoretical result that the leading
+#'bias term in nonparametric trawl function estimation is proportional to
+#'Delta times the derivative of the true trawl function.
+#'@return trawlfct Returns the original nonparametric trawl function estimate
+#'obtained from \code{\link{nonpar_trawlest}}.
+#'@return trawlfct_biascor Returns the bias-corrected trawl function estimate,
+#'computed as the original estimate minus the bias correction term
+#'\eqn{0.5 \times \Delta \times \hat{a}'(t)}.
+#'@example man/examples/trawl_est.R
+#'@seealso \code{\link{nonpar_trawlest}}, \code{\link{trawl_deriv_mod}}
+#'@export
+trawl_est <- function(data, Delta){
+
+  n<- base::length(data)
+
+  avector <- nonpar_trawlest(data, Delta=Delta, lag=n)$a_hat
+
+  #Estimate the derivative of the trawl function for the bias correction
+  deriv <- c(trawl_deriv_mod(data, Delta=Delta, lag=n-2),0) #add a 0 at the end
+  bias <- vector(mode = "numeric", length = n-1)
+
+  for(i in 1:(n-1)){
+    bias[i] <- 0.5*Delta*deriv[i]
+  }
+
+
+  avector_biascor <- avector - bias
+
+  #return list
+  return(list("trawlfct"=avector, "trawlfct_biascor"=avector_biascor))
+}
+
 #'This function estimates the size of the trawl set given by Leb(A).
 #'@title Nonparametric estimation of the trawl set Leb(A)
 #'@name LebA_est
@@ -80,7 +132,7 @@ LebA_est <- function(data, Delta, biascor=FALSE){
 
   my_avector <- avector - bias
 
-  lebA <- sum(my_avector)*Delta
+  lebA <- base::sum(my_avector)*Delta
 
   return(lebA)
 
@@ -96,6 +148,11 @@ LebA_est <- function(data, Delta, biascor=FALSE){
 #'A setdifference A_h
 #'@param biascor A binary variable determining whether a bias correction should
 #'be computed, the default is FALSE
+#'@param adjust_neg A binary variable determining whether
+#' negative estimates of the trawl function should be set to 0,
+#' the default is FALSE
+#'@param adjust_variance A binary variable determining whether a
+#'variance adjustment should be used, the default is FALSE
 #'@details Estimation of the trawl function using the methodology proposed in
 #'Sauri and Veraart (2022).
 #'@return LebA
@@ -103,13 +160,33 @@ LebA_est <- function(data, Delta, biascor=FALSE){
 #'@return LebAsetdifference
 #'@example man/examples/LebA_slice_est.R
 #'@export
-LebA_slice_est <- function(data, Delta, h, biascor=FALSE){
+LebA_slice_est <- function(data, Delta, h, biascor=FALSE,
+                           adjust_neg = FALSE,
+                           adjust_variance=FALSE){
+
+  if(h < 0) {
+    stop("h must be non-negative")
+  }
 
   n<- base::length(data)
+  avector <- nonpar_trawlest(data, Delta, lag=n)$a_hat #n-1 dimensional vec
 
-  k <- base::floor(h/Delta)
+  empirical_var <- stats::var(data)
 
-  avector <- nonpar_trawlest(data, Delta, lag=n)$a_hat
+  #Set negative values to 0 if adjustment in place
+  if(adjust_neg){
+    avector[avector<0] <- 0
+  }
+
+  scaling_factor <- 1
+
+  if(adjust_variance && base::sum(avector) > 0){
+    truncated_integral <- base::sum(avector) * Delta
+    if(truncated_integral < empirical_var && truncated_integral > 0){
+      scaling_factor <- empirical_var / truncated_integral
+      avector <- avector * scaling_factor
+    }
+  }
 
   #Estimate the derivative of the trawl function for the bias correction
   deriv <- c(trawl_deriv_mod(data, Delta=Delta, lag=n-2),0) #add a 0 at the end
@@ -123,19 +200,86 @@ LebA_slice_est <- function(data, Delta, h, biascor=FALSE){
 
   my_avector <- avector - bias
 
-  lebA <-sum(my_avector)*Delta
+  lebA <-base::sum(my_avector)*Delta
+
   #Estimate Leb(A intersection A_h)
-  lebAintersection <-sum(my_avector[(k+1):n-1])*Delta
-
-  if(lebAintersection>lebA){lebAintersection <- lebA}
-
-  lebAsetdifference <- lebA-lebAintersection
+  #Estimate Leb(A intersection A_h)
+  if(h == 0) {
+    lebAintersection <- lebA
+    lebAsetdifference <- 0
+  } else {
+    start_index <- base::ceiling(h/Delta) + 1
+    if(start_index <= base::length(my_avector)){
+      lebAintersection <- base::sum(my_avector[start_index:base::length(my_avector)]) * Delta
+    } else {
+      lebAintersection <- 0
+      message("h is too large for intersection calculation")
+    }
+    lebAsetdifference <- lebA - lebAintersection
+  }
 
   #return list
   return(list("LebA"=lebA, "LebAintersection"=lebAintersection,
               "LebAsetdifference"=lebAsetdifference))
 
 }
+################New function added in version 2.1.0
+#' This function estimates Leb(A), Leb(A intersection A_h), Leb(A\ A_h).
+#'@title Nonparametric estimation of the trawl (sub-) sets
+#'Leb(A), Leb(A intersection A_h), Leb(A setdifference A_h)
+#'@name LebA_slice_est_approx
+#'@param esttrawl Vector containing estimated trawl function.
+#'@param Delta Width of the grid on which we observe the data
+#'@param h  Time point used in A intersection A_h and the setdifference
+#'A setdifference A_h
+#'@param adjust_neg A binary variable determining whether
+#' negative estimates of the trawl function should be set to 0,
+#' the default is FALSE
+#'@details Estimation of the trawl function using the methodology proposed in
+#'Sauri and Veraart (2022).
+#'@return LebA
+#'@return LebAintersection
+#'@return LebAsetdifference
+#'@example man/examples/LebA_slice_est.R
+#'@export
+LebA_slice_est_approx <- function(esttrawl, Delta, h, adjust_neg=FALSE){
+
+  #Input validation
+  if(h < 0) {
+    stop("h must be non-negative")
+  }
+
+  n<- base::length(esttrawl) #Length of trawl function a(0),...,a((n-1)Delta)
+
+  my_avector <- esttrawl
+  #Set negative values to 0 if adjustment in place
+  if(adjust_neg){
+    my_avector[my_avector<0] <- 0
+  }
+
+  lebA <-base::sum(my_avector)*Delta
+  #Estimate Leb(A intersection A_h)
+  if(h == 0) {
+    lebAintersection <- lebA
+    lebAsetdifference <- 0
+  } else {
+    start_index <- base::ceiling(h/Delta) + 1
+    if(start_index <= base::length(my_avector)){
+      lebAintersection <- base::sum(my_avector[start_index:base::length(my_avector)]) * Delta
+    } else {
+      lebAintersection <- 0
+      message("h is too large for intersection calculation")
+    }
+    lebAsetdifference <- lebA - lebAintersection
+  }
+
+  #return list
+  return(list("LebA"=lebA, "LebAintersection"=lebAintersection,
+              "LebAsetdifference"=lebAsetdifference))
+
+}
+
+#########################
 
 #'This function estimates the ratios
 #' Leb(A intersection A_h)/Leb(A), Leb(A\ A_h)/Leb(A).
@@ -156,7 +300,7 @@ LebA_slice_ratio_est_acfbased <- function(data, Delta, h){
 
   n<- base::length(data)
 
-  k <- base::floor(h/Delta)
+  k <- base::ceiling(h/Delta)
 
   my_acf <- stats::acf(data, plot=FALSE)$acf[k+1]
 
@@ -371,7 +515,7 @@ asymptotic_variance_est <- function(t, c4, varlevyseed=1, Delta, avector,
     my_avector_length <- N+1
   }
 
-  i <- floor(t/Delta)
+  i <- base::floor(t/Delta)
 
   if(i==0){
     v1 <- c4*avector[1]
@@ -385,10 +529,10 @@ asymptotic_variance_est <- function(t, c4, varlevyseed=1, Delta, avector,
     v1  <- c4*avector[i+1]
 
     #For v2, only sum up from 1 to N+1 (corresponding to 0 to N):
-    v2 <- 2*varlevyseed^2*sum(my_avector^2)*Delta
+    v2 <- 2*varlevyseed^2*base::sum(my_avector^2)*Delta
 
     tmp3<-0
-    for(l in 0:(min(i,n-1-i))){ #Note that avector_length=n
+    for(l in 0:(base::min(i,n-1-i))){ #Note that avector_length=n
       tmp3<- tmp3+avector[i-l+1]*avector[i+l+1]
     }
     v3  <- 2*varlevyseed^2*tmp3*Delta
@@ -463,16 +607,18 @@ test_asymnorm_est_dev <- function(ahat, n, Delta, k, c4, varlevyseed=1,
 #'the modified version proposed in Sauri and Veraart (2022).
 #'@title Estimating the derivative of the trawl function
 #'@name trawl_deriv_mod
-#'@param data The data set used to compute the derivative of the trawl function
-#'@param Delta The width Delta of the observation grid
-#'@param lag The lag until which the trawl function should be estimated
+#'@param data Numeric vector of data used to compute the derivative of the
+#'trawl function
+#'@param Delta Numeric value specifying the width Delta of the observation grid
+#'@param lag Integer specifying the lag until which the trawl function
+#'derivative should be estimated. Default is 100.
 #'@details According to
 #'Sauri and Veraart (2022), the derivative of the trawl function can
 #'be estimated based on observations
 #'\eqn{X_0, X_{\Delta_n}, \ldots, X_{(n-1)\Delta_n}}  by
-#' \deqn{\widehat a(t)=\frac{1}{\sqrt{ n\Delta_{n}^2}}
+#' \deqn{\widehat a'(t)=\frac{1}{ n\Delta_{n}^2}
 #' \sum_{k=l+1}^{n-2}(X_{(k+1)\Delta_n}-X_{k\Delta_n})
-#' (X_{(k-l+1)\Delta_n}-X_{(k-l)\Delta_n}),}
+#' (X_{(k-l)\Delta_n}-X_{(k-l-1)\Delta_n}),}
 #' for \eqn{\Delta_nl\leq t < (l+1)\Delta_n}.
 #'@return The function returns the lag-dimensional vector
 #' \eqn{(\hat a'(0), \hat a'(\Delta), \ldots, \hat a'((lag-1) \Delta)).}
@@ -486,31 +632,42 @@ trawl_deriv_mod <- function(data, Delta, lag=100)
   a_deriv <- vector(mode = "numeric", length = lag)
 
   for(j in 1:lag){
-    a_deriv[j] <-base::sum(incr[(1+j):(n-1)]*incr[1:((n-1)-j)])/(n*Delta^2)
+    #a_deriv[j] <-base::sum(incr[(1+j):(n-1)]*incr[1:((n-1)-j)])/(n*Delta^2)
+    # Check if indices are valid
+    if (((n-2)-j) > 0) {
+      first_incr <- incr[(j+1):(n-2)]
+      second_incr <- incr[1:((n-2)-j)]
+      a_deriv[j] <-base::sum(first_incr*second_incr)/(n*Delta^2)
+    } else {
+      a_deriv[j] <- 0
+    }
   }
 
   return(a_deriv)
 }
 
-#'This function estimates the derivative of the trawl function using
-#'the empirical derivative of the trawl function.
-#'@title Estimating the derivative of the trawl function using the
-#'empirical derivative
-#'@name trawl_deriv
-#'@param data The data set used to compute the derivative of the trawl function
-#'@param Delta The width Delta of the observation grid
-#'@param lag The lag until which the trawl function should be estimated
-#'@details According to
-#'Sauri and Veraart (2022), the derivative of the trawl function can
-#'be estimated based on observations
-#'\eqn{X_0, X_{\Delta_n}, \ldots, X_{(n-1)\Delta_n}}  by
-#' \deqn{\widehat a(t)=\frac{1}{\Delta_{n}}
-#'(\hat a(t+\Delta_n)-\hat a(\Delta_n)),}
-#' for \eqn{\Delta_nl\leq t < (l+1)\Delta_n}.
-#'@return The function returns the lag-dimensional vector
-#' \eqn{(\hat a'(0), \hat a'(\Delta), \ldots, \hat a'((lag-1) \Delta)).}
-#'@example man/examples/trawl_deriv.R
-#'@export
+#' This function estimates the derivative of the trawl function using
+#' the empirical derivative of the trawl function.
+#' @title Estimating the derivative of the trawl function using the
+#' empirical derivative
+#' @name trawl_deriv
+#' @param data Numeric vector of data used to compute the derivative of the
+#' trawl function
+#' @param Delta Numeric value specifying the width Delta of the observation grid
+#' @param lag Integer specifying the lag until which the trawl function
+#' derivative should be estimated. Default is 100.
+#' @details This function first estimates the trawl function \eqn{\hat{a}(t)}
+#' using \code{\link{nonpar_trawlest}}, then computes the derivative using
+#' finite differences:
+#' \deqn{\widehat{a'}(l\Delta_n) = \frac{\hat{a}((l+1)\Delta_n) -
+#'  \hat{a}(l\Delta_n)}{\Delta_n},} for \eqn{l = 0, 1, \ldots, lag-1}.
+#' @return Numeric vector of length \code{lag} containing the estimated
+#' derivative values
+#' \eqn{(\hat{a'}(0), \hat{a'}(\Delta), \ldots, \hat{a'}((lag-1) \Delta))}.
+#' @seealso \code{\link{nonpar_trawlest}} for the underlying trawl function
+#' estimation
+#' @example man/examples/trawl_deriv.R
+#' @export
 trawl_deriv <- function(data, Delta, lag=100)
 {
 
@@ -612,7 +769,8 @@ test_asymnorm_est <- function(data, Delta, trawlfct, trawlfct_par,
         sqrt(n*Delta)*(avector[i]-a((i-1)*Delta)-bias[i])/sqrt(av[i])
     }
     else{
-      teststats[i] <- 999
+      teststats[i] <- Inf
+      message("Test statistic is infinite due to near-zero variance")
     }
   }
     if(biascor==TRUE){
@@ -645,7 +803,8 @@ test_asymnorm_est <- function(data, Delta, trawlfct, trawlfct_par,
         teststats <- sqrt(n*Delta)*(avector[k+1]-a(k*Delta)-bias)/sqrt(av)
       }
       else{
-        teststats <- 999
+        teststats <- Inf
+        message("Test statistic is infinite due to near-zero variance")
       }
     }
 
